/* 
 *  getit.c 2.0 (UNIX version)
 *        - use anonymous FTP to get a file named on the command line
 *        - file reference formats: 
 *           (1) host:path/file
 *           (2) path/file@host
 *           (3) ftp://host/path/file (that's the URC format)
 *
 *        - written by NASA/JSC/GM2/Ken Jenks, with help from
 *           Larry W. Virden (lwv26@cas.org)
 *
 *        - send comments to kjenks@gothamcity.jsc.nasa.gov
 *
 */

#include <stdio.h>
#include <strings.h>

#ifndef lint
static char rcsid[] = "$Header: getit.c,v 2.0 93/12/20 12:08:50 kcj Exp $";
#endif

char *status_message;
char *full_ref;
char host_name[2048],path_name[2048],file_name[2048],mail_name[2048];

extern void exit();
extern char *malloc(), *index(), *rindex();
extern int system();

main(argc, argv)
int argc;
char *argv[];
{
  FILE *fp, *fopen();

  char *p, *b, status_msg_buff[2048], *program;
  int dir, ascent, descent, error_level;

  status_message = status_msg_buff;
  *status_message = (char) NULL;

  program = argv[0];

  error_level = 0; /* Assume error code is 0 unless errors occur */

  /* 
   * If there is no file reference on command line, use false reference
   * so we give usage & exit
   */
  if(argc == 2) {
    full_ref=strncpy(malloc(strlen(argv[1])+1),argv[1],strlen(argv[1]));
    full_ref[strlen(argv[1])]=(char)NULL;
  }
  else
    full_ref = "(none)";

#ifdef DEBUG
  fprintf(stderr,"%s: full reference is %s\n",program,full_ref);
#endif

  /* Now, I'll determine the host, path and file names from the
   * full reference.
   */

  /* Case 0: full_ref = "ftp://host.path/file" (URC)
   */
  if (strstr(full_ref,"ftp://")) {

    /* Minimum is "ftp://h/f" (strlen = 9).  Must have a '/' before file.
     */
    if ((strlen(full_ref) < 9) || (index(full_ref+6,'/') == NULL)) {
      sprintf(status_message,
        "%s: can't find path or file name in %s\n",
        program,full_ref);
      error_level = 1;
      goto show_status;
    }

    strcpy(host_name,full_ref+6);
    /* Now truncate host_name at '/'
     * (We know '/' must be there 'cus of error check above.)
     */
    *index(host_name,'/') = (char)NULL;

    strcpy(path_name,index(full_ref+6,'/')+1);

    /* Case 0a: Either path has a / in it or not.
     */
    if(rindex(path_name,'/')) {
      strcpy(file_name,rindex(path_name,'/')+1);
      *rindex(path_name,'/') = (char)NULL;

    /* Case 0b: No / in path 
     */
    } else {
      strcpy(file_name, path_name);
      strcpy(path_name, ".");
    }

  /* Case 1: full_ref = "host:path/file"
   */
  } else if (index(full_ref,':')) {

    strncpy(host_name,full_ref,index(full_ref,':')-full_ref);
    strcpy(path_name,index(full_ref,':')+1);

    /* Case 1a: Either path has a / in it or not.
     */
    if(rindex(path_name,'/')) {
      strcpy(file_name,rindex(path_name,'/')+1);
      *rindex(path_name,'/') = (char)NULL;

    /* Case 1b: No / in path 
     */
    } else {
      strcpy(file_name, path_name);
      strcpy(path_name, ".");
    }

  /* Case 2: full_ref = "path/file@host"
  */
  } else if (index(full_ref,'@')) {

    strncpy(path_name,full_ref,index(full_ref,'@')-full_ref);
    strcpy(host_name,index(full_ref,'@')+1);

    /* Case 2a: Either path has a / in it or not.
    */
    if(rindex(path_name,'/')) {
      strcpy(file_name,rindex(path_name,'/')+1);
      *rindex(path_name,'/') = (char)NULL;

    /* Case 2b: No / in path 
    */
    } else {                   /* No / in path */
      strcpy(file_name, path_name);
      strcpy(path_name, ".");
    }

  /* Case 3: full_ref = none of the above
  */
  } else {
    sprintf(status_message,
      "Error: can't interpret file reference:\n%s\n\n\"%s\" provides rapid access to files using anonymous FTP.\nUsage:\n   %s host:path/file\n-or-\n   %s path/file@host\n",
      full_ref,program,program,program);
    error_level = 1;
    goto show_status;
  }

  /* Remove extraneous CR from end of strings
  */
  if(strlen(file_name))
    if(file_name[strlen(file_name)-1] == 10)
      file_name[strlen(file_name)-1] = (char)NULL;

  if(strlen(path_name))
    if(path_name[strlen(path_name)-1] == 10)
      path_name[strlen(path_name)-1] = (char)NULL;

  if(strlen(host_name))
    if(host_name[strlen(host_name)-1] == 10)
      host_name[strlen(host_name)-1] = (char)NULL;

  /* Perform the command to get user's e-mail ID and capture the results
  */
  system("echo `whoami`@`hostname`.`domainname` > getit_mail_file");

  /* Read e-mail ID for use as password in anonymous FTP
  */
  if((fp = fopen("getit_mail_file","r")) == NULL) {
    fprintf(stdout, "Warning: can't determine e-mail ID for password");
    strcpy(mail_name,"cannot-determine-user-id@getit");
  } else {
    fscanf(fp,"%s",mail_name);
  }
  strcpy(mail_name+strlen(mail_name),"(getit)");

#ifdef DEBUG
  fprintf(stderr,"  host: %s\n",host_name);
  fprintf(stderr,"  path: %s\n",path_name);
  fprintf(stderr,"  file: %s\n",file_name);
  fprintf(stderr,"  pswd: %s\n",mail_name);
#endif

  /* Open scratch file to hold FTP commands
  */
  if((fp = fopen("getit_scratch_file","w")) == NULL) {
    sprintf(status_message,"Error: can't open scratch file %s",
      "getit_scratch_file");
    error_level = 2;
    goto show_status;
  }

  /* Build the commands needed to execute the anonymous FTP
  */
  fprintf(fp,"#! /bin/csh\n");
  fprintf(fp,"/bin/echo \"Getting file %s from path %s at host %s...\"\n",
    file_name,path_name,host_name);
  fprintf(fp,"/usr/bin/ftp -inv << EOF\n");
  fprintf(fp,"open %s\n",host_name);
  fprintf(fp,"\n");
  fprintf(fp,"user anonymous %s\n",mail_name);
  fprintf(fp,"binary\n"); /* Turn binary ON for all transfers */
  fprintf(fp,"prompt\n"); /* Turn prompt OFF for possible mget */
  if((*path_name != '.') || (strlen(path_name) != 1))
    fprintf(fp,"cd %s\n",path_name);

  /* Do either mget or get, depening on existance of '*' in file_name
  */
  if(index(file_name,'*'))
    fprintf(fp,"mget %s\n", file_name); 
  else
    fprintf(fp,"get %s\n", file_name); 

  fprintf(fp,"quit\n");
  fprintf(fp,"EOF\n");
  fclose(fp);

  /* Perform the FTP and capture the results
  */
  system("sh getit_scratch_file > getit_error_file");
  system("cat getit_error_file");

show_status:
 
  system("rm -f getit_*_file > /dev/null");

  /* If there is no message to be displayed, exit quietly. 
   */
  if(status_message == NULL) (void)exit(0);
  if(strlen(status_message) == 0) (void)exit(0);
  fprintf(stderr,"%s\n",status_message);
  exit(error_level);
}
